;******************************************************************************************
;    COMPOSANT	:  ATMEL AVR 8 Bits (RISC)       AT90S2313 	Quartz = 3,6864 MHz
;    PROGRAMME	:  regul.ASM
;    VERSION	:  V1.0
;    DATE		:  15/07/00
;    DERNIERE MAJ.	:  16/07/00
;    DESCRIPTION	:  Gestion Aquastat : Temprature + Hystrsis fixe
;			:  Modification Consigne au clavier 16 touches
;
;    AUTEUR	:  Igor GODI 	Email : igor.godi@freesbee.fr
;    TESTS	:  OK
;******************************************************************************************
;			DIRECTIVES D'ASSEMBLAGE
;******************************************************************************************
	.device 	AT90S2313
	.include	"2313def.inc"	; Fichier de dfinition

;******************************************************************************************
;			CONSTANTES
;******************************************************************************************
;.equ xxxxx = xxxxxx


;******************************************************************************************
;			PORTS D'ENTREES/SORTIES
;******************************************************************************************
;________________________ CONFIGURATION PORT B __________________________________________
; Port B :	(I/O)	=	Liaison		; Description
;-------
; B_0	(Out)	=	DO LCD		; Fils rlis au bus du LCD
; B_1	(Out)	=	D1 LCD		
; B_2	(Out)	=	D2 LCD		; Et au clavier Matric
; B_3	(Out)	=	D3 LCD		
; B_4	(Out)	=	D4 LCD		
; B_5	(Out)	=	D5 LCD		
; B_6	(Out)	=	D6 LCD		
; B_7	(Out)	=	D7 LCD		
;			  76543210
.equ	CONFIGB	  =	0b11111111

;________________________ CONFIGURATION PORT D __________________________________________
; Port D :	(I/O)	=	Liaison		; Description
;-------
; D_0	(xxx)	=	xxxxxxx		; Fil xxxxxxxx
; D_1	(out)	=	Bruleur		; Fil relais bruleur
; D_2	(out)	=	Clk_DS1620	; Fil horloge DS1620
; D_3	(out)	=	Reset_DS1620	; Fil Reset DS1620
; D_4	(out)	=	Fil E_LCD	; Fil Signal Enable LCD
; D_5	(out)	=	Fil RS_LCD	; Fil Signal Register Select LCD
; D_6	(in)	=	DQ_DS1620	; Fil Donne DS1620 Sonde CDC
;			  -6543210
.equ	CONFIGD	  =	0b00111110

;******************************************************************************************
;			AUTRES CONNEXIONS ELECTRIQUES
;******************************************************************************************
; Pour Le LCD :
;	* R/W LCD  la masse
;	* +5V LCD au +5Valim
;	* Masse LCD  la masse alim

;******************************************************************************************
;			DEFINITIONS DE REGISTRES
;******************************************************************************************
; Registres  usage Global :
;-------------------------
.def DivisionClavier	= r1	; compteur de division du clavier

.def DivisionTemp	= r21	; Division du timer pour lire la T
.def TClavier	= r22		; Touche frappe au clavier
.def DRAPEAUX	= r23		; Registre drapeaux : Voir LCD.sub
.def MSB_TEMP	= r24
.def LSB_TEMP	= r25		; Poid fort et faible de la temprature
.def Div_Temp	= r26		; Diviseur pour obtenir un envoi toutes les 10s
.def Consigne	= r27		; Consigne de l'Aquastat

; Registres  usage Local :
;------------------------

.def zh		= r31		; Pointeur Z  (Utilis pour pointer en FLASH)
.def zl		= r30

; r17  r20 sont aussi utiliss en temporaire par Aff_Temp_Decimal
.def temp	= r16		; Registre temporaire


;******************************************************************************************
;			ORGANISATION RAM
;******************************************************************************************
.dseg
.org 	0x60
;xxxxx: 		byte x

;******************************************************************************************
;			ORGANISATION EEPROM
;******************************************************************************************
.eseg
.org	0x00
  ;	xxxx:	.DB	"xxxxx"


;******************************************************************************************
;			INTERRUPTIONS ET RESET
;******************************************************************************************
.cseg
.org	0x00
	rjmp 	RESET		; Vecteur RESET
	reti			; Interruption Externe 0
	reti			; Interruption Externe 1
	reti			; Capture Timer1
	rjmp	T1_COMP		; Comparaison Timer1
	reti			; Timer1 Overflow
	reti			; Timer0 Overflow
	reti			; Reception RS232 complte
	reti			; Registre emission RS232 vide
	reti			; Emission RS232 complte
	reti			; Comparateur Analogique

;******************************************************************************************
;			PROGRAMME PRINCIPAL (RESET)
;******************************************************************************************
RESET :
;__________________________________ INITIALISATIONS _______________________________________
	;----------------- Initialisation de la pile --------------------------------------
	ldi 	temp,low(RAMEND)	; Pointe en fin de mmoire
	out 	SPL,temp

	;----------------- Initialisation des ports ---------------------------------------
	ldi	temp, CONFIGB	; Init PORT B voir au dessus pour dfinition des broches
	out	DDRB,temp
	ldi	temp, CONFIGD	; Init PORT D voir au dessus pour dfinition des broches
	out	DDRD,temp

	;----------------- Initialisation des Timer (0 et 1) ------------------------------
		; TIMER 1 :
	ldi 	temp, high(0x0024)	; Poid fort registre de comparaison (si utilis)
	out	OCR1AH,temp
	ldi 	temp, low(0x0024)	; Poid faible registre de comparaison (si utilis)
	out	OCR1AL,temp		; Soit 10ms
	ldi	temp,0x00		; Utilisation de la broche de sortie 0C1 : Non
					; PWM : Arrt
	out 	TCCR1A,temp		
	ldi	temp,0x0D		; Rducteur de bruit : Non
					; Mode Comparaison (TCNT1=OCR=>RAZ) : Marche
					; Source horloge : Ck/1024=3600Hz
	out 	TCCR1B,temp		
		; INTERRUPTIONS DU TIMER
	ldi 	temp,0x40		; Timer1 : Comparaison Activ 
					
	out 	TIMSK,temp

	;----------------- Initialisation Comparateur Analogique ---------------------------
	ldi 	temp,0x80		; Comparateur Analogique : Arrt
	out	ACSR,temp

	;----------------- Initialisation des registres ------------------------------------
	ldi Consigne,70		; Init consigne  35C

	;----------------- Programmes d'Initialisation -------------------------------------
	rcall	INIT_LCD		; Initialisation Afficheur LCD
	rcall	INIT_CONFIG_LCD		; 2 lignes mode 8 bits

	rcall INIT_DS1620	; init des fils E/S du DS1620
	rcall CONFIG_DS1620	; Configuration DS1620
	rcall LANCE_CONV_DS1620	; Lance une conversion continue

;__________________________________ PROGRAMME PRINCIPAL ________________________________
	ldi zl,low(BV1*2)		; Afficher la ligne 1 Bienvenue
	ldi zh,high(BV1*2)	
	rcall Aff_Lg1

	ldi zl,low(BV2*2)		; Afficher la ligne 2 Bienvenue
	ldi zh,high(BV2*2)	
	rcall Aff_Lg2

	ldi r16,0x30			; Pause de 3s environ pour bienvenue
b1:	ldi r17,0xFF
b2:	ldi r18,0xFF
b3:	dec r18
	brne b3
	dec r17
	brne b2
	dec r16
	brne b1

	ldi zl,low(LG1*2)		; Afficher la ligne 1 Ecran
	ldi zh,high(LG1*2)	
	rcall Aff_Lg1

	ldi zl,low(LG2*2)		; Afficher la ligne 2 Ecran
	ldi zh,high(LG2*2)	
	rcall Aff_Lg2
	
	clr MSB_TEMP
	mov LSB_TEMP,Consigne		; Affiche la consigne
	ldi r16,0x85
	rcall  PROG_LCD
	rcall Aff_Temp_Decimal

	;----------------- Autorisation des intrruptions ----------------------------------
	sei			; Autorisation des interruptions aprs message bienvenue
				; et mise en place ecran de travail

PRINC:	
	rjmp PRINC


;******************************************************************************************
;			ROUTINES D'INTERRUPTIONS
;******************************************************************************************
;______________________________________ TIMER1 COMPARAISON _____________________________
T1_COMP:				; Rgl sur 10ms
					; Gestion Heure et Clavier
	push temp		; Empiler temp
	in temp,SREG
	push temp		; Empiler SREG

	;sei			; Permet de traiter d'autres interruptions
				
	;----------------- Traitement du clavier ( ttes les 100ms) -------------------------
	inc DivisionClavier	; incrmente prdiviseur Clavier
	ldi temp,10		; scrutation toutes les 10x10ms = 100ms
	cp DivisionClavier,temp
	brne Trait
	
	clr DivisionClavier	; repart  O
	rcall LitClavier	; Lecture du clavier (code Physique ou rel)
	cpi TCLavier,0x10
	breq Trait		; Si pas de touche
	cpi TClavier,0x20
	breq RetardLecture	; Si LCD Occup
	;rcall Decodage_Virtuel	; Retour du code virtuel du clavier
	rjmp PriseDecision
RetardLecture:			; Permet de rescruter le Clavier toutes les 10ms si
				; le LCD occupait le bus dans la routine en cours
				; lors de l'appel du timer 1 : permet fluidit du clavier
	ldi temp,9
	mov DivisionClavier,temp	; R-Intrrogation du clavier dans 10ms
	ldi TClavier,0x10	; permet d'viter de faire 2 tests
	rjmp Trait


	;----------------- Prises de dcisions Aprs acquistion Clavier --------------------
PriseDecision:
	cpi Tclavier,0x00	; Touche 0 ?
	brne T1			; Non Test 1
	cpi Consigne,20		; Mini 10C
	breq Trait		
	dec Consigne		; Dcrmente la consigne
	rjmp Aff_Consigne	; Affiche consigne

T1:	cpi TClavier,0x01	; Touche 1 ?
	brne T2			; Non Test 2
	cpi Consigne,140	; Maxi 70C
	breq Trait
	inc Consigne		; incrmente la consigne
Aff_Consigne:			; Affiche consigne
	clr MSB_TEMP
	mov LSB_TEMP,Consigne		; Affiche la consigne
	ldi r16,0x85
	rcall  PROG_LCD
	rcall Aff_Temp_Decimal
	rjmp Trait		; Fin d'affichage

T2:

	;----------------- Lecture T et affichage ------------------------------------------
Trait:
	inc DivisionTemp
	cpi DivisionTemp,100	; 1 seconde ?
        brne Fin_Trait		; Si non fin
	clr DivisionTemp		; Si Oui Div_Temp=0
	rcall Lit_Temp		; Lecture Temprature
	
	ldi r16,0xCB		; position Curseur
	rcall PROG_LCD		
	rcall Aff_Temp_Decimal	; Affichage de la temprature en dcimal

	;----------------- Traitement Aquastat ----------------------------------------------
	ldi r17,6		; Hystrsys/2
	mov r16,Consigne
	add r16,r17		; Ajoute Hystresis/2 (6C/2)
	cp LSB_TEMP,r16		; Temp>=Consigne+hys/2
	brge Coupe_Bruleur	; Si oui coupe le bruleur
	
	mov r16,Consigne
	subi r16,6		; Enleve Hystrsis/2
	cp r16,LSB_TEMP		; Temp<=Consigne+hys/2
	brge Allume_Bruleur	; Si oui Allume Bruleur
	rjmp Fin_Trait		; Si rien  Fin Timer

Coupe_Bruleur:
	cbi PORTD,PD1		; eteindre bruleur
	rjmp Fin_Trait

Allume_Bruleur:
	sbi PORTD,PD1		; Allumer bruleur

Fin_Trait:
	;---------------- Autres traitements temporiss -------------------------------------


Fin_Timer1:
	pop temp
	out SREG,temp		; Dpiler SREG
	pop temp		; Dpiler temp
	reti


;******************************************************************************************
;			SOUS PROGRAMMES
;******************************************************************************************
;_________________________ SOUS PROGRAMMES Gestion LCD _______________________________
.include	"LCDCLAV.sub"	; Routines de gestionnaire LCD + CLAVIER
.include "DS1620.SUB"		; Module de Gestion sonde DS1620



;******************************************************************************************
;			TABLES DE CARACTERES, TABLE ROM.....
;******************************************************************************************
BV1:	.db "*** Aquastat ***",0
BV2: 	.db "Par I.G.Software",0

LG1:	.db "Cons:xxx.x Hys:6",0
LG2: 	.db "Temp Eau : xxx.x",0
